#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netdb.h>

#include <string.h>

#include "netops.h"
#include "wrappers.h"
#include "commands.h"
#include "systool.h"


/* only used within this file, so not defined by the header */
int userauth(int sock, char *user, char *pass);


/* servattach()
 *
 * Set up the necessary data structures to attach a socket to the requested
 * server and port, attempt to connect to it, and authenticate the user.
 * Returns only the connected socket itself, or failing connection, a negative 
 * value corresponding to the failure by way of FAIL defines.
 */
int
servattach(char *server, int port, char *user, char *pass)
{
    int sock;
    struct hostent *lookup;
    struct sockaddr_in dest;


    /* fail if looking up the server fails */
    if((lookup = gethostbyname(server)) == NULL)
	return(FAILLOOKUP);

    /* fail if the request for a socket is denied */
    if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	return(FAILSOCK);

    /* set up the sockaddr struct to connect to the server */
    dest.sin_family = AF_INET;
    dest.sin_port = htons(port);
    dest.sin_addr = *((struct in_addr *)lookup->h_addr);
    bzero(&(dest.sin_zero), sizeof(dest.sin_zero));

    /* fail if connecting to this sucker fails */
    if((connect(sock, (struct sockaddr *)&dest, sizeof(struct sockaddr))) == -1)
	return(FAILCONN);

    /* attempt to authenticate the user */
    if(userauth(sock, user, pass))
       return(FAILAUTH);

    /* return the socket */
    return(sock);
}


/* sendmessage()
 *
 * Sends all of the requested message out on the specified socket, returning
 * -1 in the event of a problem.
 */
int
sendmessage(int sock, char *message)
{
    int sent, remain;


    /* find out how much to send */
    remain = strlen(message);

    /* send bits and pieces until all of it has been sent */
    while(remain)
    {
	/* check for problems on the send call */
	if((sent = send(sock, message, strlen(message), 0)) <= 0)
	{
	    /* send again on EINTR, otherwise fail */
	    if(errno == EINTR)
		sent = 0;
	    else
		return(-1);
	}

	/* move on to the next chunk of the message */
	message += sent;
	remain -= sent;
    }

    return(0);
}


/* recvmessage()
 *
 * Receive a systool network message, which is simply a load of text terminated
 * by a pair of newlines. This function currently uses Readline() and my_read()
 * from the "UNIX Network Programming" book, and should probably be rewritten
 * in a stand-alone fashion in the future.
 */
struct linelist *
recvmessage(int sock)
{
    char buff[MAXLINE];
    struct linelist *head, *curr;


    /* get ready to write to this linked list */
    head = Malloc(sizeof(struct linelist));
    curr = head;

    /* fill the list from Readline() until it returns just a newline */
    do {
	bzero(buff, sizeof(buff));
	Readline(sock, buff, MAXLINE);

	if(*(buff) != '\n')
	{
	    curr->line = Malloc(strlen(buff)+1);
	    strlcpy(curr->line, buff, (strlen(buff)+1));
	    curr->next = Malloc(sizeof(struct linelist));
	    curr = curr->next;
	}

    } while(*(buff) != '\n');

    curr->next = NULL;

    return(head);
}


/* listprofiles()
 *
 * Asks the server for a list of profiles, and returns them as a linked list
 * of lines.
 */
struct linelist *
listprofiles(int sock)
{
    char buff[MAXLINE];
    struct linelist *ret;


    snprintf(buff, sizeof(buff), "%s\n", CMD_LISTPROF);
    sendmessage(sock, buff);
    ret = recvmessage(sock);

    return(ret);
}


/* getprofile()
 *
 * Asks the server for the contents of a profile, and returns it as a linked
 * list of lines.
 */
struct linelist *
getprofile(int sock, char *profile)
{
    char buff[MAXLINE];
    struct linelist *ret;


    snprintf(buff, sizeof(buff), "%s %s\n", CMD_GETPROF, profile);
    sendmessage(sock, buff);
    ret = recvmessage(sock);

    return(ret);
}


/* getdist()
 *
 * Asks the server for the contents of a distribution, and returns it as a
 * linked list of lines.
 */
struct linelist *
getdist(int sock, char *profile, char *dist)
{
    char buff[MAXLINE];
    struct linelist *ret;


    snprintf(buff, sizeof(buff), "%s %s %s\n", CMD_GETDIST, profile, dist);
    sendmessage(sock, buff);
    ret = recvmessage(sock);

    return(ret);
}


/* userauth()
 *
 * Attempt to login to the server using the given username and password. Return
 * 0 if happy, anything else otherwise.
 */
int
userauth(int sock, char *user, char *pass)
{
    char buff[MAXLINE];
    struct linelist *temp;


    /* ignore the banner sent by the server */
    temp = recvmessage(sock);
    freelines(temp);

    /* send the login string */
    snprintf(buff, MAXLINE, "LOGIN %s\n", user);
    sendmessage(sock, buff);

    /* check out the response */
    temp = recvmessage(sock);
    chomp(&temp->line);
    if(strcmp(temp->line, "OK") != NULL)
    {
	freelines(temp);
	return(-1);
    }

    /* return success */
    freelines(temp);
    return(0);
}
