/*
 * Most of these functions are derived from 
 *
 *    Unix Network Programming Volume 1, 2nd Edition
 *    W. Richard Stevens
 *
 * Any mistakes are most likely my own, not the authors
 * - Murray Stokely
 */

#include <stdio.h>		/* perror, etc. */
#include <stdlib.h>		/* malloc */
#include <unistd.h>		/* read, write, etc.. */
#include <fcntl.h>		/* open */
#include <errno.h>
#include <ctype.h>		/* isspace */
#include <string.h>		/* strsep */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>		/* gethostbyname */

#include "wrappers.h"

#ifdef CHECKMEM
leak_t *leak_head = NULL;
#ifdef CHECKMEMSEVERE
#define DEBUG(args) printf args;
#else
#define DEBUG(args)
#endif
#else
#define DEBUG(args)
#endif


/*
 * Takes a string and seperates it based on whitespace and stores the
 * results in args.  The number of strings is returned in 'n'. 
 *
 * Example getArgs(" arg1 arg2   arg3 ", &n)
 * Will return an array with 'arg1', 'arg2', 'arg3', and n will contain '3' 
 */

char **
getArgs(char *command, int *n) {
  /* XXX evil constant */
#define MAXARGS 10
    char **args = Malloc(sizeof(char *) * MAXARGS);
    int num=0,i;
    if (command == NULL) {
	*n=num;
	return NULL;
    }
    /* Skip leading whitespace */
    for (i=0;i<MAXARGS;i++) {
	if (command == NULL) {
	    *n=num;
	    return args;
	}
	while (isspace(command[0])) command++;
	if (command == NULL) {
	    *n=num;
	    return args;
	}
	args[i] = strsep(&command, " ");
	num++;
    }
    return (args);
}

/* This function removes the end-of-line characters from a string. */

void
chomp(char **string) {
    int i;
    if (string == NULL)
	return;
    if (*string == NULL)
	return;
    for (i=0; i < strlen(*string); i++) {
	/* CRLF */
	if (((*string)[i] == 13) && ((*string)[i+1] == 10)) {
	    (*string)[i] = 0;
	    break;
	/* Just a newline */
	} else if ((*string)[i] == '\n') {
	    (*string)[i] = 0;
	    break;
	}
    }
}

/* read n bytes from a descriptor */
ssize_t
readn(int fd, void *vptr, size_t n) {
    size_t nleft;
    ssize_t nread;
    char *ptr;
    
    ptr = vptr;
    nleft = n;
    while (nleft > 0) {
	if ( (nread = read(fd,ptr,nleft)) < 0) {
	    if (errno == EINTR)
		nread = 0;
	    else
		return (-1);
	} else if (nread == 0)
	    break;			/* EOF */
	nleft -= nread;
	ptr+=nread;
    }
    return (n-nleft);		/* return >= 0 */
}

/* Writes n bytes to a socket file descriptor */
ssize_t
writen(int fd, const void *vptr, size_t n) {
    size_t nleft;
    ssize_t nwritten;
    const char *ptr;
  
    ptr = vptr;
    nleft = n;
    while (nleft>0) {
	if ( (nwritten = write(fd,ptr,nleft)) <= 0) {
	    if (errno == EINTR)
		nwritten=0;		/* and call write() again */
	    else
		return(-1);
	}
	nleft -= nwritten;
	ptr += nwritten;
    }
    return(n);
}

static ssize_t
my_read(int fd, char *ptr) {
    static int read_cnt=0;
    static char *read_ptr;
    static char read_buf[MAXLINE];
  
    if(read_cnt<=0) {
    again:
	if((read_cnt = read(fd,read_buf,sizeof(read_buf))) < 0) {
	    if (errno == EINTR)
		goto again;
	    return (-1);
	} else if (read_cnt == 0)
	    return 0;
	read_ptr=read_buf;
    }
    read_cnt--;
    *ptr=*read_ptr++;
    return(1);
}
  
ssize_t
Readline(int fd, void *vptr, size_t maxlen) {
    ssize_t n,rc;
    char c, *ptr;

    ptr=vptr;
    for (n=1;n<maxlen;n++) {
	if ( (rc=my_read(fd,&c)) == 1) {
	    *ptr++=c;
	    if (c=='\n')
		break;			/* newline is stored, like fgets */
	} else if (rc==0) {
	    if (n==1)
		return 0;		/* EOF, no data read */
	    else
		break;			/* EOF, some data was read */
	} else
	    return (-1);		/* error, errno set by read() */
    }
    *ptr=0;			/* null terminate like fgets */
    return (n);
}

int
SocketW(int family,int type,int protocol) {
    int n;
    if ( (n = socket(family, type, protocol)) < 0)
	Exit("socket");
    return (n);
}

void
Bind(int fd, const struct sockaddr *sa, socklen_t salen) {
    if (bind(fd, sa, salen) < 0)
	Exit("bind");
}

int
Connect(int fd, const struct sockaddr *sa, socklen_t salen) {
    if (connect(fd, sa, salen) < 0) {
	perror("connect");
	return -1;
    } else
	return 0;
}

int
Accept(int fd, struct sockaddr *sa, socklen_t *salenptr) {
    int n;
    if ( (n = accept(fd, sa, salenptr)) < 0) {
	Exit("accept");
    }
    return (n);
}

void
Exit(char *msg) {
    perror(msg);
    exit(1);
}

void
Write(int fd, void *ptr, size_t nbytes) {
    if (write(fd, ptr, nbytes) != nbytes)
	Exit("write");
}

ssize_t
Read(int fd, void *ptr, size_t nbytes) {
    ssize_t n;

    if ( (n = read(fd, ptr, nbytes)) == -1)
	Exit("read");
    return(n);
}

int
Open(const char *pathname, int oflag, mode_t mode) {
    int fd;

    if ( (fd = open(pathname, oflag, mode)) == -1)
	Exit("open failed");
    return(fd);
}

void *
My_Malloc(size_t size, char *filename, int linenum) {
    void    *ptr;
#ifdef CHECKMEM
    leak_t  *leak = leak_head;
#endif

    if ( (ptr = (void *)malloc(size)) == NULL)
	Exit("malloc");

    DEBUG(("My_Malloc(%p) %s:%d\n", ptr, filename, linenum));
#ifdef CHECKMEM
    if (leak_head == NULL) {
	leak_head = (leak_t *)malloc(sizeof(leak_t));
	if (leak_head == NULL)
	    Exit("malloc");
	strlcpy(leak_head->malloc_filename, filename, 256);
	leak_head->malloc_linenum = linenum;
	leak_head->next           = NULL;
	leak_head->free_filename[0] = 0;
	leak_head->free_linenum   = 0;
	leak_head->ptr            = ptr;
    } else {
	while (leak->next != NULL)
	    leak = leak->next;
	leak->next = (leak_t *)malloc(sizeof(leak_t));
	if (leak->next == NULL)
	    Exit("malloc");
	leak = leak->next;
	strlcpy(leak->malloc_filename, filename, 256);
	leak->malloc_linenum = linenum;
	leak->next           = NULL;
	leak->free_filename[0]  = 0;
	leak->free_linenum   = 0;
	leak->ptr            = ptr;
    }
#endif
    return(ptr);
}

void 
My_Free(void *ptr, char *filename, int linenum) {
#ifdef CHECKMEM
    leak_t  *leak = leak_head;

    while (leak != NULL) {
	if ((leak->ptr == ptr) && (!leak->free_linenum)) {
	    /* do stuff */
	    strlcpy(leak->free_filename, filename, 256);
	    leak->free_linenum  = linenum;
	    break;
	} 
	leak = leak->next;
    }
    if (leak == NULL) {
	printf("My_Free tried to free unused memory at %s:%d\n", filename, linenum);
	Exit("My_Free");
    }
    DEBUG(("My_Free(%p) %s:%d\n", ptr, leak->free_filename, leak->free_linenum));
#endif

    free(ptr);
}

void
Close(int fd) {
    if (close(fd) == -1)
	Exit("close");
}

void
Listen(int fd, int backlog) {
    if (listen(fd, backlog) < 0)
	perror("listen");
}

pid_t
Fork(void) {
    pid_t   pid;

    if ( (pid = fork()) == -1)
	perror("fork");
    return(pid);
}
