/*-
 * Copyright (c) 2000, 2001 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#ifdef HAVE_LIBUTIL_PROPERTY
#include <libutil.h>
#else
#include "property.h"
#endif

#include "updated.h"

/* Configuration Functions */

int
readConfig(db_t *db)
{
  int fd;
  properties propList;
  char *value = NULL;

  fd = open(".updated", O_RDONLY);
  if (fd < 0)
    return -1;

  propList = properties_read(fd);
  if (propList == NULL) 
    return -1;

  value = property_find(propList, "database_software");
  if (value == NULL) {
    printf("database_software not defined!\n");
    return -1;
  } else if (!strncmp(value, PGSQL_DESC, strlen(PGSQL_DESC)))
    db->software = T_PGSQL;
  else if (!strncmp(value, MYSQL_DESC, strlen(MYSQL_DESC)))
    db->software = T_MYSQL;
  else {
    printf("invalid entry for database_software!\n");
    return -1;
  }
  
  value = property_find(propList, "database_host");
  if (value == NULL) {
    printf("database_host not defined!\n");
    return -1;
  } else {
    SAFE_STRCPY(db->hostname, value, 256);
  }
  
  value = property_find(propList, "database_name");
  if (value == NULL) {
    printf("database_name not defined!\n");
    return -1;
  } else {
    SAFE_STRCPY(db->database, value, 256);
  }

  value = property_find(propList, "database_user");
  if (value == NULL) {
    printf("database_user not defined!\n");
    return -1;
  } else {
    SAFE_STRCPY(db->user, value, 256);
  }

  value = property_find(propList, "database_password");
  /* Password may be null, no error */
  if (value == NULL) {
    printf("pw value == NULL!\n");
    db->password[0] = '\0';
  } else
    SAFE_STRCPY(db->password, value, 16);

  value = property_find(propList, "database_port");
  if (value == NULL) {
    /* Use the default port */
    printf("No db_port, using default!\n");
    db->port = DATABASE_PORT(db->software);
  } else 
    db->port = atoi(value);

  /* Don't forget about the flags variable! */
  db->flags = 0;
  displayConfig(db);
  return 0;
}

int
configDB(db_t *db)
{
  char *answer;
  int num;
  char prompt[256];

  printf("What type of database software are you running?\n"
	 "  [%d] PostgreSQL\n"
	 "  [%d] MySQL\n", T_PGSQL, T_MYSQL);

  answer = readline("Please enter 1 or 2 [default:1]: ");
  while (answer[0] != '1' && answer[0] != '2' && answer[0] != 0) {
    free(answer);
    answer = readline("Please enter 1 or 2 : ");
  }
  if (answer[0] == 0)
    db->software = T_PGSQL;
  else
    db->software = answer[0] - 48; /* assume ASCII */
  
  snprintf(prompt, 256, "IP address of database server [%s] : ", DATABASE_HOST);
  answer = readline(prompt);
  while (ip_sane(answer)) {
    free(answer);
    answer = readline(prompt);
  }
  if (strlen(answer) > 0)
    strncpy(db->hostname, answer, sizeof(db->hostname));
  else
    strncpy(db->hostname, DATABASE_HOST, sizeof(db->hostname));

  snprintf(prompt, 256, "Database server port [%d]: ", DATABASE_PORT(db->software));
  answer = readline(prompt);
  while (ip_sane(answer)) {
    free(answer);
    answer = readline(prompt);
  }
  num = atoi(answer);
  if (num > 0)
    db->port = num;
  else
    db->port = DATABASE_PORT(db->software);

  snprintf(prompt, 256, "Database name [%s] : ", DATABASE_NAME);
  answer = readline(prompt);
  while (ip_sane(answer)) {
    free(answer);
    answer = readline(prompt);
  }
  if (strlen(answer) > 0)
    strncpy(db->database, answer, sizeof(db->database));
  else
    strncpy(db->database, DATABASE_NAME, sizeof(db->database));

  snprintf(prompt, 256, "Database username [%s] : ", DATABASE_USER);
  answer = readline(prompt);
  while (ip_sane(answer)) {
    free(answer);
    answer = readline(prompt);
  }
  if (strlen(answer) > 0) 
    strncpy(db->user, answer, sizeof(db->user));
  else
    strncpy(db->user, DATABASE_USER, sizeof(db->user));

  snprintf(prompt, 256, "Password [%s] : ", DATABASE_PASS);
  answer = readline(prompt);
  while (ip_sane(answer)) {
    free(answer);
    answer = readline(prompt);
  }
  if (strlen(answer) > 0)
    strncpy(db->password, answer, sizeof(db->password));
  else 
    strncpy(db->password, DATABASE_PASS, sizeof(db->password));

  /* Zero out anything not explicitally set */
  db->flags = 0;
  
  snprintf(prompt, 256, "Would you like to save this configuration? [y/N] : ");
  answer = readline(prompt);
  while (ip_sane(answer)) {
    free(answer);
    answer = readline(prompt);
  }
  if ((answer[0] == 'Y') || (answer[0] == 'y'))
    writeConfig(db);

  return 0;
}


void
displayConfig(db_t *db)
{
  printf("Server Software : %d\n"
	 "Server IP : %s\n"
	 "Database  : %s\n"
	 "User Name : %s\n"
	 "Password  : %s\n"
	 "Flags     : %c\n"
	 "Port      : %d\n", db->software, db->hostname, db->database,
	 db->user, db->password == NULL ? "(null)" : db->password,
	 db->flags, db->port);
}

/* Write the shit to a file or something XMLify this */
int
writeConfig(db_t *db)
{
  FILE *fp;
  fp = fopen(".updated", "w");
  if (!fp)
    return -1;
  fprintf(fp, "database_software = %s\n", DB_VERBOSE(db->software));
  fprintf(fp, "database_host = %s\n", db->hostname);
  fprintf(fp, "database_name = %s\n", db->database);
  fprintf(fp, "database_user = %s\n", db->user);
  fprintf(fp, "database_password = %s\n", db->password);
  fprintf(fp, "database_port = %d\n", db->port);
  fclose(fp);
  return 0;
}

/* Input sanity checking routines */

int ip_sane(char *ip) {
  return 0;
}

