/*-
 * Copyright (c) 2000, 2001 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <mysql.h>
#include <stdlib.h> /* malloc  */
#include <string.h> /* strncpy */
#include <syslog.h>

#include "updated.h"
#include "wrappers.h"

static long long errors=0;
int loglevel;

void print_error(MYSQL *mysql);

void
my_mysql_disconnect(void *dbp)
{
    MYSQL *mysql;
    db_t *db = dbp;
    mysql = (MYSQL *)db->handle;
    mysql_close(mysql);
    Free(mysql);
    db->handle = NULL;
    return;
}

int 
my_mysql_connect(void *dbp)
{
    MYSQL *mysql;
    db_t *db = dbp;

    /* Primitive check if connection is active */
    if (db->handle)
	return 0;

    mysql = Malloc(sizeof(MYSQL));

    /* port, options, tty */
    mysql_init(mysql);
    if(!mysql_real_connect(mysql, db->hostname, db->user, db->password, 
			   db->database, 0, 0, 0))
    {
	printf("MySQL Couldn't connect!\n");
	print_error(mysql);
	Free(mysql);
	return -1;
    }
    db->handle = mysql;
    return 0;
}

int
my_mysql_execute(void *dbp, char *query)
{
    MYSQL *mysql;
    db_t *db = dbp;
    /* wrapper in sql.c should check that this is non-null */
    mysql = (MYSQL *)db->handle;

    if (mysql_query(mysql, query)) {
	print_error(mysql);
	return -1;
    }
    return 0;
}

resultset_t *
my_mysql_select(void *dbp, char *query)
{
    MYSQL *mysql;
    MYSQL_RES *result_set=0;
    MYSQL_ROW database_row;
    MYSQL_FIELD *database_field;
    resultset_t *results;
    resultrow_t *resultrow;
    resultcell_t *resultcell;
    int current_col, cols;
    long *cellsize;
    db_t *db = dbp;
    mysql = (MYSQL *)db->handle;

    if (!mysql_query(mysql, query)) {
	if (!(result_set = mysql_store_result(mysql))) {
	    print_error(mysql);
	} else {
	    /* mysql_store_result does not return null when there are no results! */
	    if (mysql_num_rows(result_set) < 1) {
		mysql_free_result(result_set);
		return NULL;
	    }

	    results = Malloc(sizeof(resultset_t));
	    resultrow = results->rows = Malloc(sizeof(resultrow_t));
	    resultrow->next = resultrow;

	    cols = mysql_num_fields(result_set);

	    results->numfields = cols;
	    results->fields = Malloc(cols * sizeof(char *));
	    for (current_col = 0; current_col < cols; current_col++) {
		results->fields[current_col] = Malloc(16);
		database_field = mysql_fetch_field(result_set);
		strncpy(results->fields[current_col], database_field->name, 
			16);
	    }

	    /* XXX what if value is NULL? */

	    while ((database_row = mysql_fetch_row(result_set)))
	    {
		cellsize = mysql_fetch_lengths(result_set);

		/* For each row */
		resultrow = resultrow->next;
		resultcell = resultrow->cells = Malloc(sizeof(resultcell_t));
		resultcell->next = resultcell;
		for (current_col = 0; current_col < cols; current_col++) {
		    resultcell = resultcell->next;
		    resultcell->name = results->fields[current_col];
		    resultcell->datatype = STRING;
		    resultcell->value = Malloc((cellsize[current_col]+1) > 
					       MAXFIELD ? MAXFIELD : 
					       (cellsize[current_col] + 1));
		    /* XXX what if value is NULL? */

		    strncpy(resultcell->value, database_row[current_col], 
			    ((cellsize[current_col]+1) > (MAXFIELD -1) ? 
			     (MAXFIELD -1) : (cellsize[current_col]+1)));
		    resultcell->next = Malloc(sizeof(resultcell_t));
		}
		Free(resultcell->next); /* This should always be non-NULL */
		resultcell->next = NULL;
		resultrow->next = Malloc(sizeof(resultrow_t));
	    }
	    Free(resultrow->next); /* This should always be non-NULL */
	    resultrow->next = NULL;
	    mysql_free_result(result_set);
	    return results;
	}
    }
    return NULL;
}

void print_error(MYSQL *mysql) {
    errors ++;
    printf("mysql_error: %s mysql_errno: %d\n", mysql_error(mysql),
	   mysql_errno(mysql));
}
