/*-
 * Copyright (c) 2000, 2001 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <libpq-fe.h>

#include "updated.h"
#include "wrappers.h"

/* highly effective postgress index -
 * create index selects on work (probability, checkout);
 */

void
pgsql_disconnect(void *dbp)
{
    PGconn *pgsql;
    db_t *db = dbp;
    pgsql = (PGconn *)db->handle;

    PQfinish(pgsql);
}

/* try this if a connection is lost :
 * void PQreset(PGconn *conn) 
 */

int
pgsql_connect(void *dbp)
{
    PGconn *pgsql;
    db_t *db = dbp;

    /* Primitive check if connection is active */
    if (db->handle)
	return 0;

    /* port, options, tty */
    pgsql = PQsetdbLogin(db->hostname,
			 "", "", "",
			 db->database,
			 db->user,
			 db->password);
    if (pgsql == NULL) {
	/*       syslog(LOG_ERR, PQerrorMessage(pgsql)); */
	printf("Not enough memory to create login structure!\n");
	return -1;
    }
    db->handle = pgsql;
    if (PQstatus(pgsql) != CONNECTION_OK) {
	/*        syslog(LOG_ERR, PQerrorMessage(pgsql)); */
	printf("%s\n", PQerrorMessage(pgsql));
	return -1;
    }
    return 0;
}

int 
pgsql_execute(void *dbp, char *query)
{
    PGconn *pgsql;
    PGresult *res;
    db_t *db = dbp;
    pgsql = (PGconn *)db->handle;

    res = PQexec(pgsql, query);
    if (PQresultStatus(res) != PGRES_COMMAND_OK) {
	printf("%s\n", PQresStatus(PQresultStatus(res)));
	printf("%s\n", PQresultErrorMessage(res));
	return -1;
    }
    PQclear(res);
    /* Is there a way of getting the insert ID? */
    return 0;
}

resultset_t *
pgsql_select(void *dbp, char *query)
{
    PGconn *pgsql;
    PGresult *res;
    db_t *db = dbp;
    int current_row, rows, current_col, cols;
    resultset_t *results;
    resultrow_t *resultrow;
    resultcell_t *resultcell;
    int cellsize;
    pgsql = (PGconn *)db->handle;

    res = PQexec(pgsql, query);
    if (PQresultStatus(res) != PGRES_TUPLES_OK) {
	printf("%s\n", PQresStatus(PQresultStatus(res)));
	printf("%s\n", PQresultErrorMessage(res));
	return NULL;
    }
    /* We've got 0 or more results! */
    if ((rows = PQntuples(res)) == 0)
	return NULL;
    cols = PQnfields(res);

    results = Malloc(sizeof(resultset_t));
    resultrow = results->rows = Malloc(sizeof(resultrow_t));
    resultrow->next = resultrow;
    results->numfields = cols;
    results->fields = Malloc(cols * sizeof(char *));
    for (current_col = 0; current_col < cols; current_col++) {
	results->fields[current_col] = Malloc(16);
	strncpy(results->fields[current_col], PQfname(res, current_col), 16);
    }

    for (current_row = 0; current_row < rows; current_row++) {
	resultrow = resultrow->next;
	resultcell = resultrow->cells = Malloc(sizeof(resultcell_t));
	resultcell->next = resultcell;

	for (current_col = 0; current_col < cols; current_col++) {
	    resultcell = resultcell->next;
	    cellsize = PQgetlength(res, current_row, current_col);
	    resultcell->name = results->fields[current_col];
	    resultcell->datatype = STRING; 	    /* PQftype */
	    resultcell->value = Malloc(cellsize > MAXFIELD ? MAXFIELD : cellsize);
	    strncpy(resultcell->value, PQgetvalue(res, current_row, current_col), MAXFIELD);
	    resultcell->next = Malloc(sizeof(resultcell_t));
	}
	Free(resultcell->next); /* This should always be non-NULL */
	resultcell->next = NULL;
	resultrow->next = Malloc(sizeof(resultrow_t));
    }
    Free(resultrow->next); /* This should always be non-NULL */
    resultrow->next = NULL;
    PQclear(res);
    return results;
}
