/*-
 * Copyright (c) 2000, 2001 Murray Stokely, BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#define _XOPEN_SOURCE
#include <stdio.h> 
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h> 
#include <string.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <time.h>
#include <syslog.h>
#include <signal.h>

#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include "updated.h"
#include "wrappers.h"
#include "commands.h"

/* Lame, they redefine Malloc */
#include <openssl/ssl.h>

#define SA struct sockaddr
#define LISTENQ         1024    /* 2nd argument to listen() */

int loglevel = 0;
int port = 0;

#ifdef CHECKMEM
extern leak_t *leak_head;
#endif

int 
main(int argc,char **argv) {
    int c,background=0;
    db_t db;
    int listenfd, connfd;
    socklen_t len;
    struct sockaddr_in servaddr,cliaddr;
    char buff[MAXLINE];
    /*  char *msg=(char *)malloc(256); */
    
    /*  char pass[4]; */
    pid_t pid;
    
    while ((c = getopt(argc, argv, "hl:p:")) != -1) {
	/* Parse command line options */
	switch (c) {
	case 'l':
	    loglevel = atoi(optarg);
	    printf("Loglevel set to %d\n", loglevel);
	    break;
	case 'p':
	    port = atoi(optarg);
	    break;
	case 'd':
	    background = 1;
	    break;
	case 'h':
	case '?':
	default:
	    usage();
	}
    }

    if (readConfig(&db)) {
	printf("Manual configuration required!\n\n");
	if (configDB(&db)) {
	    printf("Error during manual configuration!\n");
	    return -1;
	}
    }

    if (initDB(&db))
	printf("Error initializing internal database structure!\n");

    if (!port) {
	port = DEFAULT_PORT;
	printf("No port specified, using %d\n", DEFAULT_PORT);
    }

    if (background && (daemon_init())) {
	printf("Could not become a background process!\n");
	return 1;
    }

    /* SSL_library_init() registers ciphers, we must also seed the random number */
    SSL_load_error_strings();
    SSL_library_init();
    /*    actions_to_seed_PRNG(); */


    openlog("updated", LOG_CONS|LOG_PID, LOG_LOCAL4);

    /* Initialize our new signal handlers */
    if (signal(SIGCHLD, sig_chld) == SIG_ERR)
	printf("Warning: Can't catch SIGCHLD!\n");
    if (signal(SIGINFO, sig_info) == SIG_ERR)
	printf("Warning: Can't catch SIGINFO!\n");

    /* Now we should open a socket and listen for requests. */

    listenfd = SocketW(AF_INET, SOCK_STREAM,0);
  
    bzero(&servaddr, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    servaddr.sin_port = htons(port);

    Bind(listenfd, (SA *)&servaddr, sizeof(servaddr));
    Listen(listenfd, LISTENQ);

    /* Listen for requests for eternity */
    for (;;) {
	connfd = Accept(listenfd, (SA *)&cliaddr, &len);
	syslog(LOG_ERR,"Connection from %s\n", inet_ntop(AF_INET, &cliaddr.sin_addr, buff, sizeof(buff)));

	if ( (pid = Fork()) == 0) {	/* child process */
	    Close(listenfd);		/* child closes listening socket */

	    EventLoop(connfd, &db);

	    sleep(2);
	    Close(connfd);		/* done with this client */
	    exit(0);			/* child terminates */
	}
	Close(connfd);
    }

    return 0;
}

void
EventLoop(int connfd, db_t *db) {
    SSL_CTX *ssl;
    char *command;
    char **args;
    int n;
    command = Malloc(MAXLINE);

    ssl = SSL_CTX_new(SSLv2_method());
    if (ssl == NULL) {
	printf("Could not create SSL handle!\n");
	exit(1);
    }

    if (authenticate(connfd)) {
	printf("Authentication failed!\n");
	Close(connfd);		/* done with this client */
	exit(0);		/* child terminates */
    }

    /* Read from the client while commands pending */
    while (Readline(connfd, command, MAXLINE)) {
	if (!strncmp(command, CMD_HELP, strlen(CMD_HELP))) {
	    if (strlen(command) > strlen(CMD_HELP))
		specificHelp(connfd, command + strlen(CMD_HELP));
	    else
		displayHelp(connfd);
	    Write(connfd, "\n", 1);
	} else if (!strncmp(command, CMD_LISTPROF, strlen(CMD_LISTPROF))) {
	    listProfiles(connfd, db);
	    Write(connfd, "\n", 1);
	} else if (!strncmp(command, CMD_GETPROF, strlen(CMD_GETPROF))) {
	    chomp(&command); /* remove trailing newline from command */
	    getProfile(connfd, db, command + strlen(CMD_GETPROF) + 1);
	    Write(connfd,"\n",1);
	} else if (!strncmp(command, CMD_GETDIST, strlen(CMD_GETDIST))) {
	    chomp(&command);

	    args = getArgs(command + strlen(CMD_GETDIST), &n);
	    if ( n == 2 ) {
		getDist(connfd, db, args[0], args[1]);
		Write(connfd, "\n", 1);
	    } else {
		WriteX(connfd, "ERROR: SYNAX\n");
		specificHelp(connfd, CMD_GETDIST);
	    }
	    Free(args);
	} else if (!strncmp(command, CMD_GETFILE, strlen(CMD_GETFILE))) {
	    chomp(&command);

	    args = getArgs(command + strlen(CMD_GETDIST), &n);
	    if ( n == 3 ) {
		getFile(connfd, db, args[0], args[1], args[2]);
		Write(connfd, "\n", 1);
	    } else {
		WriteX(connfd, "ERROR: SYNAX\n");
		specificHelp(connfd, CMD_GETFILE);
	    }
	    Free(args);
	} else if (!strncmp(command, CMD_QUIT, 4)) {
	    Close(connfd);
	    exit(0);
	} else {
	    WriteX(connfd, "ERROR: Unknown command\n");
	}
    }
}

void
usage() {
    printf("FreeBSD Binary Update Server v0.1\n"
	   "Options : \n"
	   "-l <loglevel>\n"
	   "-p <port>\n"
	   "-d\n");
}

/*
 * Makes this process a daemon
 */

int
daemon_init(void)
{
    pid_t pid;

    if ( (pid = fork()) < 0) {
	return(-1);
    } else if (pid!=0)
	exit(0);			/* fork and kill parent to become a daemon */

    /* child continues */
    setsid();			/* become session leader */
    chdir("/");			/* change working directory */
    umask(0);			/* clear out file creation mask */

    return(0);
}

void
sig_chld(int signo) {
    pid_t pid;
    int stat;
    pid = wait(&stat);
    return;
}

void
sig_info(int signo) {
#ifdef CHECKMEM
    leak_t *leak = leak_head;

    while (leak != NULL) {
	if ((leak->malloc_linenum) && !(leak->free_linenum))
	    printf("Memory leak from the malloc at %s:%d\n", (char *)leak->malloc_filename, leak->malloc_linenum);
	leak = leak->next;
    }
    return;
#endif
}
