/*-
 * Copyright (c) 2000 Murray Stokely
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/socket.h>

/* SQL Abstraction Structures */

typedef struct s_resultcell {
    enum { STRING, INT, BOOL } datatype;
    char *name;
    char *value;
    struct s_resultcell *next;
} resultcell_t;

typedef struct s_resultrow {
    struct s_resultcell *cells;
    struct s_resultrow *next;
} resultrow_t;

typedef struct s_resultset {
    int numfields;
    char **fields;
    resultrow_t *rows;
} resultset_t;

typedef int (SQLProc) (void *db);
typedef int (ExecProc) (void *db, char *query);
typedef resultset_t *(SelectProc) (void *db, char *query);

typedef struct s_db {
    char hostname[256];
    char database[256];
    int software;
    char user[256];
    char password[16];
    int  port;
    char flags;
    void *handle;
    SQLProc *connect;
    SQLProc *disconnect;
    ExecProc *execute;
    SelectProc *select;
} db_t;

/* Reasonable default values */

#define DATABASE_NAME "updates"
#define DATABASE_USER "updates"
#define DATABASE_HOST "localhost"
#define DATABASE_PASS ""
#define PGSQL_PORT 5432
#define MYSQL_PORT 3306
#define T_PGSQL 1
#define T_MYSQL 2
#define PGSQL_DESC "PGSQL"
#define MYSQL_DESC "MYSQL"
#define DB_VERBOSE(type) type == 1 ? PGSQL_DESC : MYSQL_DESC
#define DATABASE_PORT(software) software == 1 ? PGSQL_PORT : MYSQL_PORT

#define DEFAULT_PORT 31337
#define MAXLINE 256
#define MAXQUERY 256
#define HUGEQUERY 2048
#define MAXFIELD 256

#ifdef HAVE_STRLCPY
#define SAFE_STRCPY(dest, src, destlen) strlcpy(dest, src, destlen)
#else
#warning "strncpy being used since you don't have strlcpy.  may not NUL-terminate strings"
#define SAFE_STRCPY(dest, src, destlen) strncpy(dest, src, destlen)
#endif

#define AUTH_MSG "The authentication module has not been written yet.\nFor the time being the client should simply send 'LOGIN BSDI\\n'\n"
#define DEFAULT_SERVER "127.0.0.1"


#define socklen_t uint32_t

extern int loglevel; /* How detailed should we be with the logging? */

/* #define LOG_QUERY(query) if (loglevel > 1) printf("%s\n",query); else if (loglevel > 0) syslog(LOG_DEBUG,query) */
#define LOG_QUERY(query) if (loglevel > 0) syslog(LOG_DEBUG,query)
#define CHECKED 1

/* Authentication Prototypes */
int authenticate(int fd);
void EventLoop(int connfd, db_t *db);
int daemon_init(void);
void listProfiles(int fd, db_t *db);
void getProfile(int fd, db_t *db, char *profile);
void getDist(int fd, db_t *db, char *profile, char *dist);
void getFile(int connfd, db_t *db, char *profile, char *dist, char *file);

void usage(void);
void sig_int(int signo);
void sig_chld(int signo);
void sig_info(int signo);

/* SQL stuff */
int initDB(db_t *db);
char *sql_getvalue(resultrow_t *resultrow, char *colname);
void sql_freeresults(resultset_t *results);
void sql_freerow(resultrow_t *resultrow);
void sql_freecells(resultcell_t *resultcell);

#ifdef HAVE_PGSQL
int pgsql_connect(void *db);
void pgsql_disconnect(void *db);
int pgsql_execute(void *db, char *query);
resultset_t *pgsql_select(void *dbp, char *query);
#endif
#ifdef HAVE_MYSQL
int my_mysql_connect(void *db);
void my_mysql_disconnect(void *db);
int my_mysql_execute(void *db, char *query);
resultset_t *my_mysql_select(void *dbp, char *query);
#endif

/* Configuration functions */
int configDB(db_t *db);
int ip_sane(char *ip);
int readConfig(db_t *db);
int writeConfig(db_t *db);
void displayConfig(db_t *db);

/* Help functions */
void displayHelp(int fd);
void specificHelp(int fd, char *cmd);

/* Prototypes for the wrapper functions */
/*
ssize_t Readline(int fd, void *vptr, size_t maxlen);
int SocketW(int family,int type,int protocol);
void Bind(int fd, const struct sockaddr *sa, socklen_t salen);
int Accept(int fd, struct sockaddr *sa, socklen_t *salenptr);
int Connect(int fd, const struct sockaddr *sa, socklen_t salen);
void Close(int fd);
void Listen(int fd, int backlog);
ssize_t Read(int fd, void *ptr, size_t nbytes);
void Write(int fd, void *ptr, size_t nbytes);
int Open(const char *pathname, int oflag, mode_t mode);
void Exit(char *msg);
void *Malloc(size_t size);
pid_t Fork(void);
*/
